import random
import json
# Memory
good_answers = {}
bad_answers = {}

THRESHOLD = max(10_000, 100_000 // (1 + len(good_answers)))
MAX_BYTES = 2048


def save(filename):
    with open(filename + ".respmem", "w") as f:
        json.dump({'good': good_answers, 'bad': bad_answers,'threshold': THRESHOLD, 'maxbytes': MAX_BYTES}, f)


import json

def load(filename):
    global good_answers, bad_answers, THRESHOLD, MAX_BYTES
    with open(filename + ".respmem", "r") as f:
        data = json.load(f)
        good_answers = {int(k): v for k, v in data["good"].items()}
        bad_answers = {int(k): v for k, v in data["bad"].items()}
        THRESHOLD = data.get("threshold", THRESHOLD)  # default fallback
        MAX_BYTES = data.get("maxbytes", MAX_BYTES)



def str_to_bin(s):
    return ''.join(format(ord(c), '08b') for c in s)


def bin_to_str(b):
    chars = [b[i:i + 8] for i in range(0, len(b), 8)]
    return ''.join(chr(int(char, 2)) for char in chars if len(char) == 8)


def bin_to_int(b):
    return int(b, 2)


def int_to_bin(i, pad_to=0):
    b = bin(i)[2:]
    return b.zfill(pad_to) if pad_to else b


def get_similar_input_response(input_val):
    for past_input in good_answers:
        diff = abs(past_input - input_val)
        if diff <= THRESHOLD:
            return good_answers[past_input]
    return None


def generate_random_output():
    length = random.randint(1, MAX_BYTES)
    return ''.join(format(random.randint(32, 126), '08b') for _ in range(length))


def ai_respond(input_bin):
    input_val = bin_to_int(input_bin)
    if input_val in good_answers:
        return good_answers[input_val]
    similar = get_similar_input_response(input_val)
    if similar and random.choice([True, False]):
        return similar
    return generate_random_output()


def main():
    global good_answers, bad_answers
    while True:
        user_input = input(f"\nEnter ASCII text (max {MAX_BYTES} bytes): ")

        if user_input.lower() == "exit":
            filename = input("Enter filename to save intelligence: ")
            if filename=="_TEMP":
                break
            save(filename)
            print("Memory saved. Goodbye!")
            break

        if user_input.lower() == "load":
            filename = input("Enter saved filename (without extension): ")
            try:
                load(filename)
                print("Memory loaded.")
            except FileNotFoundError:
                print("File not found.")
            continue

        if len(user_input.encode('utf-8')) > MAX_BYTES:
            print("Input too long.")
            continue

        input_bin = str_to_bin(user_input)
        input_val = bin_to_int(input_bin)
        ai_bin_output = ai_respond(input_bin)

        try:
            ai_text_output = bin_to_str(ai_bin_output)
        except Exception:
            ai_text_output = "<Invalid ASCII Output>"

        print(f"AI Output: {ai_text_output}")
        feedback = input("Was this a good output? (yes/no): ").strip().lower()

        if feedback == "yes":
            good_answers[input_val] = ai_bin_output
            print("Stored in good_answers.")
        else:
            bad_answers[input_val] = ai_bin_output
            print("Stored in bad_answers.")
        global THRESHOLD
        THRESHOLD = max(10_000, 100_000 // (1 + len(good_answers)))


if __name__ == "__main__":
    MAX_BYTES=int(input("What should be max I/O bytes?(default 2048) "))
    main()
